import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:spreadsheet_decoder/spreadsheet_decoder.dart';

Future<void> main() async {
  WidgetsFlutterBinding.ensureInitialized();
  // await autoUpdateAT();
  // await autoUpdateBE();
  await autoUpdateDE();
  // await autoUpdateLT();
  // await autoUpdateLU();
  // await autoUpdateNL();
  debugPrint('Finished.');
}

Future<void> autoUpdateAT() async {
  final List<List<String>> table = await _loadCSV('at');
  Map<String, Iban> result = <String, Iban>{};
  for (int i = 6; i < table.length; i++) {
    final String bic = table[i][18];
    final String bankCode = table[i][2];
    final String bankName = table[i][6];
    final String bankAddress = table[i][7];
    final String bankPostcode = table[i][8];
    final String bankLocation = table[i][9];
    result['"$bankCode"'] = Iban(
      bic: bic,
      bankName: bankName,
      bankShortName: bankName,
      bankAddress: bankAddress,
      bankPostcode: bankPostcode,
      bankLocation: bankLocation,
    );
  }
  Clipboard.setData(ClipboardData(text: '$result'));
}

Future<void> autoUpdateBE() async {
  // https://www.nbb.be/doc/be/be/protocol/r_fulllist_of_codes_current.xlsx
  final SpreadsheetTable table = await _loadSpreadsheet('be');
  Map<String, Iban> result = <String, Iban>{};
  for (int i = 2; i < table.rows.length; i++) {
    final String bic = table.rows[i][1];
    final String bankCode = table.rows[i][0];
    final String bankName = table.rows[i][2];
    result['"$bankCode"'] = Iban(
      bic: bic,
      bankName: bankName,
      bankShortName: bankName,
    );
  }
  Clipboard.setData(ClipboardData(text: '$result'));
}

Future<void> autoUpdateDE() async {
  // https://www.bundesbank.de/resource/blob/602630/7892d0ad8bbd16165b87e92195bfc88f/mL/blz-aktuell-xlsx-data.xlsx
  final SpreadsheetTable table = await _loadSpreadsheet('de');
  Map<String, Iban> result = <String, Iban>{};
  String lastBic = '';
  for (int i = 1; i < table.rows.length; i++) {
    final String bankCode = table.rows[i][0];
    final String bankName = table.rows[i][2];
    final String bankShortName = table.rows[i][5];
    final String bankPostcode = table.rows[i][3];
    final String bankLocation = table.rows[i][4];
    String? bic = table.rows[i][7];
    if (bic == null || bic.isEmpty) {
      bic = lastBic;
    } else {
      lastBic = bic;
    }
    result['"$bankCode"'] = Iban(
      bic: bic,
      bankName:
          bankName.length < bankShortName.length ? bankShortName : bankName,
      bankShortName:
          bankName.length > bankShortName.length ? bankShortName : bankName,
      bankPostcode: bankPostcode,
      bankLocation: bankLocation,
    );
  }
  Clipboard.setData(ClipboardData(text: '$result'));
}

Future<void> autoUpdateLT() async {
  // https://www.lb.lt/uploads/documents/files/LT-20221020-en.xlsx
  final SpreadsheetTable table = await _loadSpreadsheet('lt');
  Map<String, Iban> result = <String, Iban>{};
  for (int i = 1; i < table.rows.length; i++) {
    final String bic = table.rows[i][1];
    final String bankCode = table.rows[i][0].toString();
    final String bankName = table.rows[i][2];
    final String? bankAddress = table.rows[i][5];
    final String bankPostcode = table.rows[i][6].toString();
    final String bankLocation = table.rows[i][5];
    result['"$bankCode"'] = Iban(
      bic: bic,
      bankName: bankName,
      bankShortName: bankName,
      bankAddress: bankAddress,
      bankPostcode: bankPostcode,
      bankLocation: bankLocation,
    );
  }
  Clipboard.setData(ClipboardData(text: '$result'));
}

Future<void> autoUpdateLU() async {
  // https://abbl.lu/media/file/global/dynamic/5ef5523ce29390f43972a6390f1e1893b7c2f789/Luxembourg%20Register%20of%20IBAN-BIC%20Codes-01.10.2022.xlsx
  final SpreadsheetTable table = await _loadSpreadsheet('lu');
  Map<String, Iban> result = <String, Iban>{};
  for (int i = 3; i < table.rows.length; i++) {
    if (table.rows[i][1] == null ||
        table.rows[i][1].toString().trim().isEmpty) {
      continue;
    }
    final String bic = table.rows[i][2];
    final String bankCode = table.rows[i][1].toString();
    final String bankName = table.rows[i][0];
    result['"$bankCode"'] = Iban(
      bic: bic,
      bankName: bankName,
      bankShortName: bankName,
    );
  }
  Clipboard.setData(ClipboardData(text: '$result'));
}

Future<void> autoUpdateNL() async {
  // https://www.betaalvereniging.nl/wp-content/uploads/BIC-lijst-NL.xlsx
  final SpreadsheetTable table = await _loadSpreadsheet('nl');
  Map<String, Iban> result = <String, Iban>{};
  for (int i = 4; i < table.rows.length; i++) {
    final String? bic = table.rows[i][0];
    if (bic == null) {
      continue;
    }
    final String bankCode = table.rows[i][1];
    final String bankName = table.rows[i][2];
    result['"$bankCode"'] = Iban(
      bic: bic,
      bankName: bankName,
      bankShortName: bankName,
    );
  }
  Clipboard.setData(ClipboardData(text: '$result'));
}

Future<List<List<String>>> _loadCSV(String name) async {
  final String data = await rootBundle.loadString('res/$name.csv');
  final List<List<String>> result = <List<String>>[];
  for (String line in data.split('\n')) {
    result.add(line.split(';'));
  }
  return result;
}

Future<SpreadsheetTable> _loadSpreadsheet(String name) async {
  final Uint8List bytes =
      (await rootBundle.load('res/$name.xlsx')).buffer.asUint8List();
  final SpreadsheetDecoder decoder = SpreadsheetDecoder.decodeBytes(bytes);
  return decoder.tables[decoder.tables.keys.first]!;
}

class Iban {
  final String bic;
  final String bankName;
  final String bankShortName;
  final String? bankAddress;
  final String? bankPostcode;
  final String? bankLocation;

  const Iban({
    required this.bic,
    required this.bankName,
    required this.bankShortName,
    this.bankAddress,
    this.bankPostcode,
    this.bankLocation,
  });

  @override
  String toString() {
    return 'Iban('
        'bic: "${bic.replaceAll('"', '\\"')}", '
        'bankName: "${bankName.replaceAll('"', '\\"')}", '
        'bankShortName: "${bankShortName.replaceAll('"', '\\"')}", '
        '${bankAddress != null ? 'bankAddress: "${bankAddress!.replaceAll('"', '\\"')}", ' : ''}'
        '${bankPostcode != null ? 'bankPostcode: "${bankPostcode!.replaceAll('"', '\\"')}", ' : ''}'
        '${bankLocation != null ? 'bankLocation: "${bankLocation!.replaceAll('"', '\\"')}",' : ''})';
  }
}
